C      PROGRAM PLF5 BY MIKE FOLSE  LAST MODIFIED 10/24/94
C      LINEAR STATIC  ANALYSIS OF PLANE FRAMES AND TRUSSES
C       USE CONSISTENT UNITS
C
C       NODE NUMBERS WILL BE TEXT PLOTTED
C
C      INPUT FROM FILE PLF5.DAT OR FROM KEYBOARD
C       YOU MAY BUILD PLF5.DAT IN AN EDITOR OR WITHIN PROGRAM
C       PROGRAM AUTOMATICALLY STORES KEYBOARD INPUT IN PLF5.DAT
C      OUTPUT IS TO FILE PLF5.OUT
C
C      INPUT LIST FOR FILE PLF5.DAT(FREE FORMAT-SEPARATE INPUT
C           ITEMS WITH ONE COMMA OR ONE OR MORE BLANKS,INPUT
C           ALL ITEMS EVEN IF ZERO)
C
C      LINE 1T : WRITE "TITLE"
C      LINE 1: TITLE(15A4)
C      LINE 2T: WRITE "NJT,NMEM,NLDJT,E,NMWR,NXTS,NYTS"
C      LINE 2: NUMBER OF JOINTS(NJT), (150MAX)
C            NUMBER OF MEMBERS(NMEM), (200MAX)
C            NUMBER OF JOINTS WITH APPLIED LOADS(NLDJT),
C            MODULUS OF ELASTICITY(E),
C            NUMBER OF MEMBERS WITH END RELEASES (NMWR)
C               NMWR=-1 INDICATES ALL MEMBERS PINNED BOTH ENDS
C                      IE. A TRUSS
C          NUMBER OF JTS WITH SPECIFIED X TRANSLATIONS(NXTS)(10MAX)
C          NUMBER OF JTS WITH SPECIFIED Y TRANSLATIONS(NYTS)(10MAX)
C      LINE 3T: WRITE "JT NUM,X,Y,NS/SX,SY,SRZ
C             (2ND LINE ONLY IF NS=1)"
C      LINE 3 (REPEAT NJT TIMES): JOINT NUMBER, X COORDINATE,
C             Y COORDINATE, SUPPORT CODE(NS)
C         (=1 IF AT LEAST 1 DOF AT JOINT IS SUPPORT,0 OTHERWISE)
C         NOTE:INPUT JOINTS IN ASCENDING ORDER,MISSING JOINTS WILL
C               BE GENERATED BASED ON EQUAL SPACING
C      LINE 3A  IF NS=1,INPUT SPRING SUPPORT CONSTANTS IN X,Y, AND
C              Z ROTATION.
C            IF NEGATIVE NUMBER INPUT,DEFAULT = 1.E20 ASSIGNED
C      LINE 4T WRITE "MEM NUM,J,K,A,I,MLC/WJ,WK,NCL
C                     /LD,DIS,LD,DIS,LD,DIS"
C      LINE 4 (REPEAT NMEM TIMES): MEMBER NUMBER, JOINT NUMBER OF
C             JOINT TO BE CONSIDERED J NODE THIS MEMBER,
C             JOINT NUMBER OF K NODE THIS MEMBER,
C             CROSS-SECTION AREA THIS MEMBER, 
C             MOMENT OF INERTIA ABOUT BENDING AXIS,
C             MEMBER LOAD CODE(MLC)
C               (=1 IF MEMBER HAS LOAD, =0 OTHERWISE)
C      LINE 4A   IF MLC=1 INPUT VALUE OF DISTRIBUTED LOAD AT J NODE
C     THEN VALUE AT K NODE, THEN COUNTERCLOCKWISE ANGLE IN DEGREES
C       FROM PERPENDICULAR TO THE DIRECTION OF DIST LOAD,
C     THEN NUMBER OF CONCENTRATED LOADS, 5 MAX (CONCENTRATED LOADS
C       MUST BE PERPENDICULAR TO MEMBER (ANGLE = 0).
C       FOR NON-PERPENDICULAR CONCENTRATED LOADS PUT A JOINT AT
C         LOAD LOCATION
C        INPUT LOAD IN POSITIVE LOCAL Y DIRECTION AS POSITIVE
C      LINE 4B IF THERE ARE CONC LOADS,INPUT( LOAD VALUE,
C              DISTANCE FROM J) FOR EACH LOAD
C      LINE 5T :(IF NLDJT .NE. 0) WRITE " JT NUM,XF,YF,CCZM"
C      LINE 5 (REPEAT NLDJT TIMES): JOINT NUMBER OF LOADED JOINT,
C             APPLIED LOAD IN X DIRECTION,
C             APPLIED LOAD IN Y DIRECTION,
C             APPLIED COUNTERCLOCKWISE MOMENT ABOUT Z AXIS
C   LINE 6T (IF NMWR.NE.0)WRITE "MEMBERS, END RELEASE CODE"
C    LINE 6(REPEAT NMWR TIMES)
C             MEMBER NUMBER
C           RELEASE CODE,NP = 0  NO RELEASES
C                         = 1  PINNED BOTH ENDS,NO AXIAL RELEASE
C                         = 2  PINNED J END ONLY,NO AXIAL RELEASE
C                         = 3  PINNED K END ONLY,NO AXIAL RELEASE
C                         = 4  PINNED BOTH ENDS,AXIAL RELEASE AT J
C                         = 5  PINNED BOTH ENDS,AXIAL RELEASE AT K
C                         = 6  PINNED J END, AXIAL RELEASE AT J
C                         = 7  PINNED K END, AXIAL RELEASE AT K
C   LINE 7T (IF (NXTS .NE. 0)
C          WRITE "JOINT, SPECIFIED X TRANSLATION"
C   LINE 7 (REPEAT NXTS TIMES)
C              JOINT, SPECIFIED X TRANSLATION
C   LINE 8T (IF (NYTS .NE. 0)
C          WRITE "JOINT, SPECIFIED Y TRANSLATION"
C   LINE 8 (REPEAT NYTS TIMES)
C              JOINT, SPECIFIED Y TRANSLATION
       CALL INPUT
       CALL ANALYSIS
       CALL OUTPUT
       STOP
       END
C
C
      SUBROUTINE INPUT
      DIMENSION TITLE(15),AJ(60),NSX(150),NSY(150)
      COMMON/AA/ A(450,63),B(450)
      REAL*8 A,B
      COMMON/BB/IOUT,NJT,NMEM,E,NMWR,NXTS,NYTS,IB
      COMMON/CC/X(150),Y(150),NS(150),BJT(450),
     1 JEND(200),KEND(200),RI(200),
     1 AX(200),ALEN(200),CXX(200),CYY(200),MLC(200),
     1 WL(200),WR(200),THETAD(200),NP(200),
     1 NCLD(200),PV(200,5),PD(200,5),
     1 NJXT(10),SXT(10),NJYT(10),SYT(10)
      COMMON/DD/SM(6,6),SMRT(6,6),SMR(6,6),RT(6,6),RTRANS(6,6),
     1 IN(6),EACT(6),FEM(6),P(6),D(6)
      COMMON/EE/SX(150),SY(150),SRZ(150)
      CHARACTER LLINE*63,DUM*1,AJ*2,ANUM*2
      DATA AJ/' 1',' 2',' 3',' 4',' 5',' 6',' 7',' 8',' 9','10',
     1 '11','12','13','14','15','16','17','18','19','20',
     1 '21','22','23','24','25','26','27','28','29','30',
     1 '31','32','33','34','35','36','37','38','39','40',
     1 '41','42','43','44','45','46','47','48','49','50',
     1 '51','52','53','54','55','56','57','58','59','60'/
      OPEN(6,FILE='PLF5.OUT',STATUS='UNKNOWN')
      CLOSE(6,STATUS='DELETE')
      OPEN(6,FILE='PLF5.OUT',STATUS='NEW')
C      PROGRAM DIMENSIONED FOR MAXIMUM 150 JOINTS AND 200 MEMBERS
      JDIM=63
   30 CONTINUE
      NCD=0
      IB=0
C     MAXIMUM DIFFERENCE IN END NODE NUMBERS=20
      IP=5
      IOUT=6
      WRITE(*,5000)
 5000 FORMAT(5X,'SELECT INPUT MODE',//,15X,'(1)  KEYBOARD',
     1//,15X,'(0)  FILE PLF5.DAT',//,15X,'(2)  TERMINATE',/)
      READ(*,*)NCD
      IF(NCD.NE.0.AND.NCD.NE.1)STOP
      WRITE(*,5005)
 5005 FORMAT(/,5X,'OUTPUT IS TO FILE PLF5.OUT',//,5X,
     1 ' "EDIT PLF5.OUT" TO VIEW CONTENTS',//,5X,
     1 ' "PRINT PLF5.OUT" TO GET PRINTED OUTPUT')
      IF(NCD.EQ.1)GO TO 50
      OPEN(5,FILE='PLF5.DAT',STATUS='OLD',ACCESS='SEQUENTIAL')
      READ(IP,1002)TITLE
 1002 FORMAT(/,15A4,/)
      GO TO 60
   50 CONTINUE
      OPEN(5,FILE='PLF5.DAT',STATUS='UNKNOWN')
      CLOSE(5,STATUS='DELETE')
      OPEN(5,FILE='PLF5.DAT',STATUS='NEW',ACCESS='SEQUENTIAL')
      WRITE(*,5010)
 5010 FORMAT(/,5X,'ENTER TITLE UP TO 60 CHARACTERS',/)
      READ(*,1000)TITLE
      WRITE(IP,1001)TITLE
 1001 FORMAT(1X,'TITLE',/,1X,15A4)
 1000 FORMAT(15A4)
   60 CONTINUE
      WRITE(IOUT,2000)TITLE
 2000 FORMAT(//,10X,15A4)
      IF(NCD.EQ.1)GO TO 80
      READ(IP,*)NJT,NMEM,NLDJT,E,NMWR,NXTS,NYTS
      READ(IP,'(A1)')DUM
      GO TO 85
   80 CONTINUE
      WRITE(*,5020)
 5020 FORMAT(/,5X,'ENTER NUMBER OF JOINTS(150 MAXIMUM)',/)
      READ(*,*)NJT
      WRITE(*,5021)
 5021 FORMAT(/,5X,'ENTER NUMBER OF MEMBERS(200 MAXIMUM)',/)
      READ(*,*)NMEM
      WRITE(*,5022)
 5022 FORMAT(/,5X,'ENTER NUMBER OF JOINTS WITH APPLIED LOADING',
     1 /)
      READ(*,*)NLDJT
      WRITE(*,5023)
 5023 FORMAT(/,5X,'ENTER MODULUS OF ELASTICITY(USE CONSISTENT',
     1 ' UNITS)',/)
      READ(*,*)E
      WRITE(*,5025)
 5025 FORMAT(/,5X,'ENTER NUMBER OF MEMBERS WITH END RELEASES',
     1 /,10X,'VALUE OF -1 INDICATES ALL PINNED(A TRUSS)',/)
      READ(*,*)NMWR
      WRITE(*,7025)
 7025 FORMAT(/,5X,'ENTER NUMBER OF SPECIFIED X TRANSLATIONS',
     1' (10MAX)',/)
      READ(*,*)NXTS
      WRITE(*,7026)
 7026 FORMAT(/,5X,'ENTER NUMBER OF SPECIFIED Y TRANSLATIONS',
     1' (10MAX)',/)
      READ(*,*)NYTS
      WRITE(IP,1010)NJT,NMEM,NLDJT,E,NMWR,NXTS,NYTS
 1010 FORMAT(' NJT,NMEM,NLDJT,E,NMWR,NXTS,NYTS',/,3I5,E14.5,3I5,
     1/,' JT NUM,X,Y,NS/SX,SY,SRZ(2ND LINE ONLY IF NS=1)')
      WRITE(*,5024)
 5024 FORMAT(///,5X,'INPUT JOINT COORDINATES AND SPRING ',
     1 'SUPPORTS',//)
   85 CONTINUE
      WRITE(IOUT,2010)NJT,NMEM,NLDJT,E,NMWR,NXTS,NYTS
 2010 FORMAT(//,10X,'NUMBER OF JOINTS =',I5,
     1 /,10X,'NUMBER OF MEMBERS =',I5,
     1 /,10X,'NUMBER OF JOINTS WITH APPLIED LOAD(S) =',I5,
     1/,10X,'MODULUS OF ELASTICITY =',E14.5,
     1/,10X,'NUMBER OF MEMBERS WITH END RELEASES =',I5,
     1/,20X, ' ( -1 INDICATES TRUSS)',
     1/,10X,'NUMBER OF SPECIFIED X TRANSLATIONS =',I5,
     1/,10X,'NUMBER OF SPECIFIED Y TRANSLATIONS =',I5,
     1 //,40X,'SPRING CONSTANTS',/,1X,'JOINT        X',
     1 12X,'Y            SX          SY          SRZ',/)
      NUMEQ=3*NJT
      DO 87 I=1,NJT
      SX(I)=0.
      SY(I)=0.
      SRZ(I)=0.
   87 CONTINUE
      MM=0
   88 CONTINUE
      IF(NCD.EQ.1)GO TO 90
      READ(IP,*)M,X(M),Y(M),NS(M)
      IF(NS(M).EQ.1)READ(IP,*)SX(M),SY(M),SRZ(M)
      GO TO 100
   90 CONTINUE
      WRITE(*,5030)  
 5030 FORMAT(/,2X,'ENTER JOINT NUMBER,X COORD,',
     1'Y COORD AND SUPPORT CODE(SC=1 IF ANY SUPPORTS)',/)
      READ(*,*)M,X(M),Y(M),NS(M)
      IF(NS(M).EQ.1)WRITE(*,5031)
 5031 FORMAT(/,5X,'ENTER SPRING SUPPORT CONSTANTS X,Y,Z',
     1/,15X,'(EX. -1,-1,-1 DENOTES ALL THREE DOF FIXED)',/)
      IF(NS(M).EQ.1)READ(*,*)SX(M),SY(M),SRZ(M)
      WRITE(IP,1020)M,X(M),Y(M),NS(M)
 1020 FORMAT(I5,2E14.5,I5)
      IF(NS(M).EQ.1)WRITE(IP,1021)SX(M),SY(M),SRZ(M)
 1021 FORMAT(6E13.4,/,6E13.4)
  100 CONTINUE
      IF(SX(M).LT.0.)SX(M)=1.E30
      IF(SY(M).LT.0.)SY(M)=1.E30
      IF(SRZ(M).LT.0.)SRZ(M)=1.E30
      IF(M.EQ.(MM+1))GO TO 116
C     GENERATE MISSING NODES
      M1=MM+1
      M2=M-1
      DIFF=FLOAT(M-MM)
      XINC=(X(M)-X(MM))/DIFF
      YINC=(Y(M)-Y(MM))/DIFF
      AINC=0.
      DO 115 K=M1,M2
      AINC=AINC+1.
      X(K)=X(MM)+AINC*XINC
      Y(K)=Y(MM)+AINC*YINC
C      NOTE:GENERATED NODES CANNOT BE SUPPORTS
      NS(K)=0
  115 CONTINUE
  116 IF(M.EQ.NJT)GO TO 119
      MM=M
      GO TO 88
  119 CONTINUE
      DO 120 I=1,NJT
  120 WRITE(IOUT,2020)I,X(I),Y(I),SX(I),SY(I),SRZ(I)
 2020 FORMAT(1X,I5,5E13.4)
C
      WRITE(IOUT,2082)
 2082 FORMAT(///,5X,'PLOT OF JOINTS',//)
C     DETERMINE MAXIMUM AND MINIMUM X AND Y COORDINATES
      XMAX=X(1)
      XMIN=X(1)
      YMAX=Y(1)
      YMIN=Y(1)
      DO 125 I = 2,NJT
      IF(X(I).LT.XMIN)XMIN=X(I)
      IF(X(I).GT.XMAX)XMAX=X(I)
      IF(Y(I).LT.YMIN)YMIN=Y(I)
      IF(Y(I).GT.YMAX)YMAX=Y(I)
  125 CONTINUE
      XRANG=XMAX-XMIN
      IF(XRANG.LT.1.)XRANG=1.
      YRANG=YMAX-YMIN
      IF(YRANG.LT.1.)YRANG=1.
C     SCALE TO 62 SPACES BY 20 LINES
      DO 127 I=1,NJT
      NSX(I)=IFIX((X(I)-XMIN)/XRANG*62.)
      NSY(I)=IFIX((Y(I)-YMIN)/YRANG*20.)
      IF(NSX(I).EQ.0)NSX(I)=1
      IF(NSY(I).EQ.0)NSY(I)=1
  127 CONTINUE
      DO 129 I= 1,20
      LLINE(1:30)= '..............................'
      LLINE(31:63)='.................................'
      DO 128 J= 1,NJT
      IF(NSY(J).NE.(21-I))GO TO 128
      ID1=NSX(J)
      ID2=NSX(J)+1
      ANUM=AJ(J)
      LLINE(ID1:ID2)=ANUM
  128 CONTINUE
      WRITE(IOUT,3389)LLINE
 3389 FORMAT(3X,A63)
  129 CONTINUE
      IF(NCD.EQ.0)READ(IP,'(A1)')DUM
      IF(NCD.EQ.1)WRITE(IP,2023)
 2023 FORMAT(' MEM NUM,J,K,A,I,MLC/WJ,WK,THETAD,NCL',
     1'/LD,DIS,LD,DIS,LD,DIS')
      DO 140 I=1,NMEM
      NP(I)=0
      DO 131 J=1,3
      PV(I,J)=0.
      PD(I,J)=0.
  131 CONTINUE
      IF(NCD.EQ.1)GO TO 130
      READ(IP,*)M,JEND(M),KEND(M),AX(M),RI(M),MLC(M)
      IF(MLC(M).EQ.0)GO TO 135
      READ(IP,*)WL(M),WR(M),THETAD(M),NCLD(M)
      NCL=NCLD(M)
      IF(NCLD(M).GT.0)READ(IP,*)(PV(M,J),PD(M,J),J=1,NCL)
      GO TO 135
  130 CONTINUE
      WRITE(*,5040)
 5040 FORMAT(/,5X,'ENTER MEM NUM,J NODE,K NODE,AREA,INER,LD CODE',
     1/,10X,'(LD CODE =1 IF ANY MEMBER LOADS)',/)
      READ(*,*)M,JEND(M),KEND(M),AX(M),RI(M),MLC(M)
      WRITE(IP,1030)M,JEND(M),KEND(M),AX(M),RI(M),MLC(M)
 1030 FORMAT(3I5,2E14.5,I5)
      IF(MLC(M).EQ.0)GO TO 135
      WRITE(*,5042)
 5042 FORMAT(/,2X,'ENTER: DISTRIBUTED LOAD VALUE J END',/,
     1 9X,'DISTRIBUTED LOAD VALUE K END',/,
     1 9X,'CCW ANGLE IN DEGREES FROM PERPENDICULAR TO DIRECTION',/,
     1 16X,'OF DISTRIBUTED LOAD',/,
     1 9X,'NUMBER OF CONCENTRATED LOADS (5 MAXIMUM)',//,
     1 6X,'NOTE: DISTRIBUTED LOAD WITH POSITIVE LOCAL Y (UP)',/,
     1 14X,'COMPONENT INPUT AS POSITIVE WITH ANGLE BETWEEN',/,
     1 14X,'-90 DEGREES AND +90 DEGREES',/)
      READ(*,*)WL(M),WR(M),THETAD(M),NCLD(M)
      WRITE(IP,6021)WL(M),WR(M),THETAD(M),NCLD(M)
 6021 FORMAT(3E14.5,I5)
      NCL=NCLD(M)
      IF(NCL.EQ.0)GO TO 135
      WRITE(*,5043)
 5043 FORMAT(/,2X,'ENTER: LOAD VALUE',/,9X,
     1 'DISTANCE FROM J NODE',//,10X, 
     1 'FOR UP TO 5 CONCENTRATED LOADS',//,8X,
     1 'NOTE: CONCENTRATED MEMBER LOADS MUST BE PERPENDICULAR',/,
     116X,'TO THE MEMBER, LOADS IN +Y LOCAL (UP) INPUT POSITIVE',/)
      READ(*,*)(PV(M,J),PD(M,J),J=1,NCL)
      WRITE(IP,1021)(PV(M,J),PD(M,J),J=1,NCL)
  135 CONTINUE
C
C     CALCULATE BAND WIDTH OF STIFFNESS MATRIX
      NBW=3*(IABS(JEND(M)-KEND(M))+1)
      IF(NBW.GT.IB)IB=NBW
  140 CONTINUE
      IF(IB.LE.JDIM)GO TO 150
      WRITE(IOUT,2025)IB
 2025 FORMAT(/,10X,'BAND WIDTH EXCEEDS DIMENSION ALLOWED',
     1 'IB=',I5)
  150 CONTINUE
      WRITE(IOUT,2030)
 2030 FORMAT(//,5X,'MEMBER   J NODE    K NODE    AREA',
     18X, 'INERTIA    LENGTH')
      DO 160 I=1,NMEM
      JJ=JEND(I)
      KK=KEND(I)
      XC=X(KK)-X(JJ)
      YC=Y(KK)-Y(JJ)
      AL=SQRT(XC*XC+YC*YC)
  160 WRITE(IOUT,2040)I,JEND(I),KEND(I),AX(I),RI(I),AL
 2040 FORMAT(3(5X,I5),4E13.4)
      WRITE(IOUT,2035)
 2035 FORMAT(//,5X,'MEMBER LOADS',/,2X,'MEMBER    W AT J',
     1'     W AT K    THETAD          P       D')
      DO 170 I=1,NMEM
      IF(MLC(I).EQ.0)GO TO 170
      NCL=NCLD(I)
      WRITE(IOUT,2041)I,WL(I),WR(I),THETAD(I),
     1 (PV(I,J),PD(I,J),J=1,NCL)
  170 CONTINUE
 2041 FORMAT(1X,I5,2X,3E11.3,5(/,40X,2E11.3))
      IF(IB.GT.JDIM)STOP
C     INITIALIZE STIFFNESS AND LOAD MATRICES TO ZERO
      DO 180 I=1,NUMEQ
      B(I)=0.
      BJT(I)=0.
      DO 180 J=1,IB
      A(I,J)=0.
  180 CONTINUE
      IF(NLDJT.EQ.0)GO TO 191
C     INPUT JOINT LOADS
      WRITE(IOUT,2045)
 2045 FORMAT(//,5X,'JOINT      XFORCE    YFORCE      ZMOM',/)
      IF(NCD.EQ.0)READ(IP,'(A1)')DUM
      IF(NCD.EQ.1)WRITE(IP,2046)
 2046 FORMAT(' JT NUM,XF,YF,CCZM')
      DO 190 I=1,NLDJT
      IF(NCD.EQ.1)GO TO 185
      READ(IP,*)JT,XFORCE,YFORCE,ZMOM
      GO TO 188
  185 CONTINUE
      WRITE(*,5050)
 5050 FORMAT(/,5X,'ENTER JOINT,FORCE X DIRECTION,',
     1 'FORCE Y DIRECTION, AND CC MOMENT',/)
      READ(*,*)JT,XFORCE,YFORCE,ZMOM
      WRITE(IP,1040)JT,XFORCE,YFORCE,ZMOM
 1040 FORMAT(I5,3E12.3)
  188 CONTINUE
      WRITE(IOUT,2050)JT,XFORCE,YFORCE,ZMOM
 2050 FORMAT(5X,I5,3E13.4)
C     ADD FORCES TO APPLIED LOAD MATRIX
      NXDF=3*JT-2
      NYDF=NXDF+1
      NZDF=NXDF+2
C     CONVERT TO DOUBLE PRECISION
      B(NXDF)=DBLE(XFORCE)
      B(NYDF)=DBLE(YFORCE)
      B(NZDF)=DBLE(ZMOM)
      BJT(NXDF)=XFORCE
      BJT(NYDF)=YFORCE
      BJT(NZDF)=ZMOM
  190 CONTINUE
  191 CONTINUE
      IF(NMWR.LE.0)GO TO 210
      DO 207 I =1,NMWR
      IF(NCD.EQ.0.AND.I.EQ.1)READ(IP,'(A1)')DUM
      IF(NCD.EQ.1.AND.I.EQ.1)
     1   WRITE(IP,*)' MEMBER NUMBER, END RELEASE CODE'
      IF(NCD.EQ.1)GO TO 200
      READ(IP,*)MEM,NP(MEM)
      GO TO 205
  200 CONTINUE
      WRITE(*,5051)
 5051 FORMAT(/,5X,'ENTER MEMBER NUMBER AND END RELEASE CODE',/,
     112X,'0 = NO RELEASES',/,12X,'1 = PINNED BOTH ENDS',/,12X,
     1 '2 = PINNED J END ONLY',/,12X,'3 = PINNED K END ONLY',/,12X,
     1 '4 = PINNED BOTH ENDS,AXIAL RELEASE AT J',/,12X,
     1 '5 = PINNED BOTH ENDS,AXIAL RELEASE AT K',/,12X,
     1 '6 = PINNED J END, AXIAL RELEASE AT J',/,12X,
     1 '7 = PINNED K END, AXIAL RELEASE AT K',/)
      READ(*,*)MEM,NP(MEM)
      WRITE(IP,5055)MEM,NP(MEM)
 5055 FORMAT(2I5)
  205 CONTINUE
      IF(I.EQ.1) WRITE(IOUT,5056)
 5056 FORMAT(//,5X,'MEMBER     RELEASE CODE',/)
      WRITE(IOUT,5057)MEM,NP(MEM)
 5057 FORMAT(5X,I5,8X,I5)
  207 CONTINUE
  210 CONTINUE
      IF(NXTS.EQ.0)GO TO 710
      IF(NCD.EQ.0)READ(IP,'(A1)')DUM
      IF(NCD.EQ.1)
     1  WRITE(IP,*)' JOINT NUMBER, SPECIFIED X TRANSLATION'
      IF(NCD.EQ.1)GO TO 700
      READ(IP,*)(NJXT(I),SXT(I),I=1,NXTS)
      GO TO 705
  700 CONTINUE
      DO 701 I=1,NXTS
      WRITE(*,7051)
 7051 FORMAT(/,5X,'ENTER JOINT NUMBER AND SPECIFIED X',
     1' TRANSLATION',/)
      READ(*,*)NJXT(I),SXT(I)
      WRITE(IP,7055)NJXT(I),SXT(I)
 7055 FORMAT(I5,E11.3)
  701 CONTINUE
  705 CONTINUE
      WRITE(IOUT,7056)
 7056 FORMAT(//,5X,'JOINT    SPECIFIED X TRANSLATION',/)
      DO 707 I=1,NXTS
      WRITE(IOUT,7057)NJXT(I),SXT(I)
 7057 FORMAT(5X,I5,10X,E11.3)
  707 CONTINUE
  710 CONTINUE
      IF(NYTS.EQ.0)GO TO 810
      IF(NCD.EQ.0)READ(IP,'(A1)')DUM
      IF(NCD.EQ.1)
     1  WRITE(IP,*)' JOINT NUMBER, SPECIFIED Y TRANSLATION'
      IF(NCD.EQ.1)GO TO 800
      READ(IP,*)(NJYT(I),SYT(I),I=1,NYTS)
      GO TO 805
  800 CONTINUE
      DO 801 I=1,NYTS
      WRITE(*,8051)
 8051 FORMAT(/,5X,'ENTER JOINT NUMBER AND SPECIFIED Y',
     1' TRANSLATION',/)
      READ(*,*)NJYT(I),SYT(I)
      WRITE(IP,7055)NJYT(I),SYT(I)
  801 CONTINUE
  805 CONTINUE
      WRITE(IOUT,8056)
 8056 FORMAT(//,5X,'JOINT    SPECIFIED Y TRANSLATION',/)
      DO 807 I=1,NYTS
      WRITE(IOUT,7057)NJYT(I),SYT(I)
  807 CONTINUE
  810 CONTINUE
      RETURN
      END
C
C
      SUBROUTINE ANALYSIS
      COMMON/BB/IOUT,NJT,NMEM,E,NMWR,NXTS,NYTS,IB
      COMMON/DD/SM(6,6),SMRT(6,6),SMR(6,6),RT(6,6),RTRANS(6,6),
     1 IN(6),EACT(6),FEM(6),P(6),D(6)
      COMMON/EE/SX(150),SY(150),SRZ(150)
      COMMON/CC/X(150),Y(150),NS(150),BJT(450),
     1 JEND(200),KEND(200),RI(200),
     1 AX(200),ALEN(200),CXX(200),CYY(200),MLC(200),
     1 WL(200),WR(200),THETAD(200),NP(200),
     1 NCLD(200),PV(200,5),PD(200,5),
     1 NJXT(10),SXT(10),NJYT(10),SYT(10)
      COMMON/AA/ A(450,63),B(450)
      REAL*8 A,B
C     FORM MEMBER STIFFNESS MATRICES, ROTATE TO GLOBAL COORDINATES,
C       INDEX TO GLOBAL DEGREE OF FREEDOM NUMBERS, AND ADD TO
C       GLOBAL STIFFNESS MATRIX
      NUMEQ=3*NJT
      DO 300 I=1,NMEM
      IF(NMWR.LT.0)NP(I)=1
      JJ=JEND(I)
      KK=KEND(I)
      XC=X(KK)-X(JJ)
      YC=Y(KK)-Y(JJ)
      AL=SQRT(XC*XC+YC*YC)
      CX=XC/AL
      CY=YC/AL
      CXX(I)=CX
      CYY(I)=CY
C     CX,CY, ARE DIRECTION COSINES FOR MEMBER I, I.E. COSINES OF
C      ANGLES BETWEEN MEMBER AXIS AND GLOBAL X,Y AXES
      ALEN(I)=AL
      CALL FORMRT(RT,RTRANS,CX,CY)
      CALL FORMSM(SM,AL,AX(I),E,RI(I),NP(I))
      CALL MULT(SM,RT,6,SMRT)
      CALL MULT(RTRANS,SMRT,6,SMR)
      CALL INDX(JJ,KK,IN)
C     FORM UPPER HALF OF STIFFNESS MATRIX,STORE IN RECTANGULAR
C     ARRAY WITH DIAGONAL ELEMENT IN COLUMN 1
      DO 260 J=1,6
      JG=IN(J)
      DO 260 K=1,6
      KG=IN(K)
      IF(KG.LT.JG)GO TO 260
      LG=KG-JG+1
C     CONVERT TO DOUBLE PRECISION
      A(JG,LG)=A(JG,LG)+DBLE(SMR(J,K))
  260 CONTINUE
C     ADD MEMBER END ACTIONS TO LOAD MATRIX
      IF(MLC(I).EQ.0)GO TO 300
      CALL FM(WL(I),WR(I),THETAD(I),AL,FEM,NP(I))
      NCL=NCLD(I)
      IF(NCL.EQ.0)GO TO 265
      DO 263 J=1,NCL
  263 CALL FM2(PV(I,J),PD(I,J),AL,FEM,NP(I))
  265 CALL MULT(RTRANS,FEM,1,P)
C STORE FIXED END ACTIONS AT J IN PV AND FIXED ACTIONS AT K INPD
      PV(I,1)=FEM(1)
      PV(I,2)=FEM(2)
      PV(I,3)=FEM(3)
      PD(I,1)=FEM(4)
      PD(I,2)=FEM(5)
      PD(I,3)=FEM(6)
      DO 270 J=1,6
      B(IN(J))=B(IN(J))-DBLE(P(J))
  270 CONTINUE
  300 CONTINUE
C     CORRECT GLOBAL STIFFNESS MATRIX FOR BOUNDARY CONDITIONS
C       I.E. SUPPORT SPRINGS WHERE INPUT
      DO 360 I=1,NJT
      NXDF=3*I-2
      NYDF=NXDF+1
      NZDF=NXDF+2
      IF(NS(I).EQ.0)GO TO 350
      A(NXDF,1)=A(NXDF,1)+DBLE(SX(I))
      A(NYDF,1)=A(NYDF,1)+DBLE(SY(I))
      A(NZDF,1)=A(NZDF,1)+DBLE(SRZ(I))
  350 IF(NMWR.EQ.-1)A(NZDF,1)=1.E20
  360 CONTINUE
C     CORRECT FOR SPECIFIED X TRANSLATIONS
      IF(NXTS.EQ.0) GO TO 362
      DO 361 I =1,NXTS
      NDOF=3*NJXT(I)-2
      A(NDOF,1)=1.E28
      B(NDOF)=1.E28*SXT(I)
  361 CONTINUE
  362 IF(NYTS.EQ.0)GO TO 364
      DO 363 I=1,NYTS
      NDOF=3*NJYT(I)-1
      A(NDOF,1)=1.E28
      B(NDOF)=1.E28*SYT(I)
  363 CONTINUE
  364 CONTINUE
C     SOLVE   AX=B FOR DISPLACEMENT MATRIX X
      CALL UDU(NUMEQ,IB,NER)
      IF(NER.EQ.0)GO TO 370
      WRITE(IOUT,4200)
 4200 FORMAT(/,10X,'STIFFNESS MATRIX IS SINGULAR')
      STOP
  370 CONTINUE
C     NOTE: DEFLECTIONS ARE RETURNED IN B MATRIX WHICH WENT INTO
C         UDU AS APPLIED LOADS
      RETURN
      END
C
      SUBROUTINE OUTPUT
      DIMENSION SUM(450),DL(6),DLS(200,6)
      COMMON/AA/A(450,63),B(450)
      REAL*8 A,B
      COMMON/BB/IOUT,NJT,NMEM,E,NMWR,NXTS,NYTS,IB
      COMMON/DD/SM(6,6),SMRT(6,6),SMR(6,6),RT(6,6),RTRANS(6,6),
     1 IN(6),EACT(6),FEM(6),P(6),D(6)
      COMMON/CC/X(150),Y(150),NS(150),BJT(450),
     1 JEND(200),KEND(200),RI(200),
     1 AX(200),ALEN(200),CXX(200),CYY(200),MLC(200),
     1 WL(200),WR(200),THETAD(200),NP(200),
     1 NCLD(200),PV(200,5),PD(200,5),
     1 NJXT(10),SXT(10),NJYT(10),SYT(10)
C     INITIALIZE REACTION MATRIX
      NUMEQ=3*NJT
      DO 100 I=1,NUMEQ
      SUM(I)=0.
  100 CONTINUE
      WRITE(IOUT,4100)
 4100 FORMAT(//,2X,'OUTPUT',//,
     1 9X,'JOINT DISPLACEMENTS ',//,5X,'JT'
     1,'   X TRANSLATION    Y TRANSLATION  Z ROTATION,RADIANS')
      DO 400 I=1,NJT
      NXDF=3*I-2
      NYDF=NXDF+1
      NZDF=NXDF+2
      WRITE(IOUT,4110)I,B(NXDF),B(NYDF),B(NZDF)
 4110 FORMAT(2X,I5,2X,3(D11.3,6X))
  400 CONTINUE
      WRITE(IOUT,4115)
 4115 FORMAT(//,13X,'MEMBER END ACTIONS',//,1X,'MEMBER J   ',
     1'K     AX J     SHR J      MOM J        AX K     SHR K', 
     1 6X,'MOM K')
      DO 500 I=1,NMEM
      AL=ALEN(I)
      JJ=JEND(I)
      KK=KEND(I)
      CALL INDX(JJ,KK,IN)
      DO 450 J=1,6
C     CONVERT TO SINGLE PRECISION
  450 D(J)=SNGL(B(IN(J)))
      CALL FORMRT(RT,RTRANS,CXX(I),CYY(I))
      CALL FORMSM(SM,AL,AX(I),E,RI(I),NP(I))
      CALL MULT(RT,D,1,DL)
C     SAVE MEMBER DEFORMATIONS
      DO 455 J=1,6
      DLS(I,J)=DL(J)
  455 CONTINUE
C      CALL MULT(SM,RT,6,SMRT)
C      CALL MULT(SMRT,D,1,EACT)
      CALL MULT(SM,DL,1,EACT)
C     ADD FIXED END ACTIONS
      IF(MLC(I).EQ.0)GO TO 480
      DO 460 K=1,3
  460 EACT(K)=EACT(K)+PV(I,K)
      DO 470 K=4,6
      KM3=K-3
  470 EACT(K)=EACT(K)+PD(I,KM3)
  480 CONTINUE
      WRITE(IOUT,4120)I,JJ,KK,(EACT(J),J=1,6)
 4120 FORMAT(1X,3I4,6E11.3)
C     ROTATE END ACTIONS TO GLOBAL COORDINATES FOR RECREATION
C      OF JOINT LOADS AND REACTIONS
      CALL MULT(RTRANS,EACT,1,P)
      DO 482 J=1,6
      SUM(IN(J))=SUM(IN(J))+P(J)
  482 CONTINUE
  500 CONTINUE
C     OUTPUT MEMBER DEFORMATIONS
      WRITE(IOUT,4210)
 4210 FORMAT(//,2X,'MEMBER DEFORMATIONS IN LOCAL COORDINATES',/,
     15X,'(-0.999E+22 INDICATES DEFORMATION NOT COMPUTED DUE TO',
     1' END RELEASES)',//,' MEMBER   XTJ         YTJ  ',
     1'    ZRJ         XTK        YTK        ZRK',/)
      CON=-0.999E+22
      DO 600 I=1,NMEM
      NPM=NP(I)
      IF(NPM.EQ.0)GO TO 580
      IF(NPM.NE.1.AND.NPM.NE.4.AND.NPM.NE.5)GO TO 510
      DLS(I,3)=CON
      DLS(I,6)=CON
  510 CONTINUE
      IF(NPM.EQ.2.OR.NPM.EQ.6)DLS(I,3)=CON
      IF(NPM.EQ.3.OR.NPM.EQ.7)DLS(I,6)=CON
      IF(NPM.EQ.4.OR.NPM.EQ.6)DLS(I,1)=CON
      IF(NPM.EQ.5.OR.NPM.EQ.7)DLS(I,4)=CON
  580 CONTINUE
      WRITE(IOUT,4212)I,(DLS(I,J),J=1,6)
 4212 FORMAT(1X,I5,6E11.3)
  600 CONTINUE
C     SUBTRACT JOINT LOAD MATRIX FROM SUMMATION OF END ACTIONS
      DO 710 I=1,NUMEQ
      SUM(I)=SUM(I)-BJT(I)
  710 CONTINUE
C     OUTPUT REACTIONS
      WRITE(IOUT,4230)
 4230 FORMAT(//,5X,'REACTIONS RECREATED FROM',
     1' MEMBER END ACTIONS',
     1 /,5X,'JOINT     XFORCE      YFORCE      ZMOMENT',/)
      DO 720 I =1,NJT
      NXDF=3*I-2
      NYDF=NXDF+1
      NZDF=NXDF+2
      WRITE(IOUT,4231)I,SUM(NXDF),SUM(NYDF),SUM(NZDF)
 4231 FORMAT(5X,I5,3E12.3)
  720 CONTINUE
      RETURN
      END
C
C
C     SUBROUTINE UDU
      SUBROUTINE UDU(NUMEQ,IB,NER)
C     UDU  FOR BANDED SYMMETRIC OR  MATRICES
C      BY ERIK THOMPSON OF CSU
      COMMON/AA/A(450,63),X(450)
      REAL*8 A,X,FAC
      NER=0
      NEM1=NUMEQ-1
      DO 450 I=1,NEM1
      JEND=NUMEQ-I+1
      IF(JEND.GT.IB)JEND=IB
      DO 440 J=2,JEND
      J1=J+I-1
      IF(DABS(A(I,1)).LT.0.0000001)GO TO 480
      FAC=A(I,J)/A(I,1)
      K1=0
      DO 430 K=J,JEND
      K1=K1+1
      A(J1,K1)=A(J1,K1)-FAC*A(I,K)
  430 CONTINUE
      A(I,J)=FAC
  435 CONTINUE
      X(J1)=X(J1)-A(I,J)*X(I)
  440 CONTINUE
  450 CONTINUE
      X(NUMEQ)=X(NUMEQ)/A(NUMEQ,1)
      DO 470 I=1,NEM1
      I1=NUMEQ-I
      X(I1)=X(I1)/A(I1,1)
      JEND=NUMEQ-I1+1
      IF(JEND.GT.IB)JEND=IB
      DO 460 J=2,JEND
      J1=I1+J-1
      X(I1)=X(I1)-A(I1,J)*X(J1)
  460 CONTINUE
  470 CONTINUE
      RETURN
  480 NER=1
      RETURN
      END
      SUBROUTINE FORMSM(SM,AL,AX,E,RI,NP)
C     STIFFNESS MATRIX PLANE FRAME OR TRUSS
      DIMENSION SM(6,6)
      DO 10 I = 1,6
      DO 10 N = I,6
      SM(I,N)=0.
      A=AX*E/AL
C     IF NP.GT.3,THERE IS AN AXIAL RELEASE
      IF(NP.GT.3)A=0.
   10 SM(N,I)=0.0
      SM(1,1)=A
      SM(4,4)=A
      SM(1,4)=-A
      SM(4,1)=-A
      SM(3,3)=0.0001*4.*E*RI/AL
      SM(6,6)=SM(3,3)
      IF(NP.EQ.1.OR.NP.EQ.4.OR.NP.EQ.5)RETURN
      CON=12.
C     IF EITHER END PINNED (NOT BOTH), CON =3.
      IF(NP.GT.1)CON=3.
      SM(2,2)=CON*E*RI/AL**3
      SM(5,5)=SM(2,2)
      SM(2,5)=-SM(2,2)
C     NP = 2 OR 6 MEANS PINNED ON J END (DOF 3)
      IF(NP.EQ.2.OR.NP.EQ.6)GO TO 50
C     NP = 3 OR 7 MEANS PINNED ON K END (DOF 6)
      IF(NP.EQ.3.OR.NP.EQ.7)GO TO 100
C     NEITHER END PINNED
      SM(2,3)=6.*E*RI/AL**2
      SM(2,6)=SM(2,3)
      SM(3,5)=-SM(2,3)
      SM(5,6)=SM(3,5)
      SM(3,3)=4.*E*RI/AL
      SM(6,6)=SM(3,3)
      SM(3,6)=.5*SM(3,3)
      GO TO 200
   50 CONTINUE
C     LEFT END PINNED
      SM(2,6)=3.*E*RI/AL**2
      SM(5,6)=-SM(2,6)
      SM(6,6)=3.*E*RI/AL
      GO TO 200
  100 CONTINUE
C     RIGHT END PINNED
      SM(2,3)=3.*E*RI/AL**2
      SM(3,3)=3.*E*RI/AL
      SM(3,5)=-SM(2,3)
  200 CONTINUE
      DO 20 I=1,5
      J=I+1
      DO 20 M=J,6
   20 SM(M,I)=SM(I,M)
      RETURN
      END
      SUBROUTINE FORMRT(RT,RTRANS,CX,CY)
C     FORM ROTATION MATRICES
      DIMENSION RT(6,6),RTRANS(6,6)
      DO 10 I = 1,6
      DO 10 N = I,6
      RT(I,N)=0.
   10 RT(N,I)=0.0
      RT(1,1)=CX
      RT(2,2)=CX
      RT(4,4)=CX
      RT(5,5)=CX
      RT(1,2)=CY
      RT(4,5)=CY
      RT(2,1)=-CY
      RT(5,4)=-CY
      RT(3,3)=1.
      RT(6,6)=1.
C      FORM RTRANS
      DO 30 I = 1,6
      DO 30 N = 1,6
   30 RTRANS(N,I)=RT(I,N)
      RETURN
      END
      SUBROUTINE MULT(A,B,JB,C)
C      MATRIX MULTIPLY
      DIMENSION A(6,6),B(6,JB),C(6,JB)
      DO 10 I =1,6
      DO 10 N =1,JB
      SUM=0.0
      DO 20 L=1,6
   20 SUM=SUM+A(I,L)*B(L,N)
   10 C(I,N)=SUM
      RETURN
      END
      SUBROUTINE INDX(JJ,KK,IN)
      DIMENSION IN(6)
C     INDX GIVES GLOBAL DEGREE OF FREEDOM NUMBERS CORRESPONDING
C      TO THE SIX MEMBER LOCAL DEGREES OF FREEDOM 
      J3=3*JJ
      K3=3*KK
      IN(3)=J3
      IN(2)=J3-1
      IN(1)=J3-2
      IN(6)=K3
      IN(5)=K3-1
      IN(4)=K3-2
      RETURN
      END
      SUBROUTINE FM(WL,WR,THETAD,AL,FEM,NP)
C     SUBROUTINE TO CALC FIXED END ACTIONS FOR LINEAR LOAD
      DIMENSION FEM(6)
      THETAR=THETAD/57.3
      WLN=WL*COS(THETAR)
      WLPAR=WL*SIN(THETAR)
      WRN=WR*COS(THETAR)
      WRPAR=WR*SIN(THETAR)
C     POSITIVE PARALLEL COMPONENT ACTS TOWARD J NODE
C     NP=0 NO RELEASES
C     NP=1 PINNED BOTH ENDS, NO AXIAL RELEASE
C     NP=2 PINNED J END, NO AXIAL RELEASE
C     NP=3 PINNED K END, NO AXIAL RELEASE
C     NP=4 PINNED BOTH ENDS, AXIAL RELEASE AT J
C     NP=5 PINNED BOTH ENDS, AXIAL RELEASE AT K
C     NP=6 PINNED J END, AXIAL RELEASE AT J
C     NP=7 PINNED K END, AXIAL RELEASE AT K
      FEM(1)=1.*(0.5*AL*(WLPAR*0.667+WRPAR*0.333))
      FEM(4)=1.*(0.5*AL*(WLPAR*0.333+WRPAR*0.667))
      IF(NP.EQ.4.OR.NP.EQ.6)FEM(4)=FEM(4)+FEM(1)
      IF(NP.EQ.4.OR.NP.EQ.6)FEM(1)=0.
      IF(NP.EQ.5.OR.NP.EQ.7)FEM(1)=FEM(1)+FEM(4)
      IF(NP.EQ.5.OR.NP.EQ.7)FEM(4)=0.
      IF(NP.EQ.1.OR.NP.EQ.4.OR.NP.EQ.5)GO TO 100
      FEM(3)=-1./12.*WLN*AL**2-1./30.*(WRN-WLN)*AL**2
      FEM(6)=1./12.*WLN*AL**2+1./20.*(WRN-WLN)*AL**2
      IF(NP.EQ.0)GO TO 110
      IF(NP.EQ.3.OR.NP.EQ.7)GO TO 90
C     J END PINNED
      FEM(6)=FEM(6)-0.5*FEM(3)
      FEM(3)=0.
      GO TO 110
   90 CONTINUE
C     K END PINNED
      FEM(3)=FEM(3)-0.5*FEM(6)
      FEM(6)=0.
      GO TO 110
  100 CONTINUE
      FEM(3)=0.
      FEM(6)=0.
  110 CONTINUE
      FEM(2)=-WLN*AL*.5-(WRN-WLN)*AL*.5/3.+(FEM(3)+FEM(6))/AL
      FEM(5)=-WLN*AL*.5-(WRN-WLN)*AL*.333333-(FEM(3)+FEM(6))/AL
      RETURN
      END
      SUBROUTINE FM2(PV,PD,AL,FEM,NP)
C     SUBROUTINE TO CALCULATE FIXED END ACTIONS DUE TO A
C       CONCENTRATED LOAD PERPENDICULAR TO THE MEMBER,
C       AND ADD TO PREVIOUS VALUES OF FIXED END ACTIONS
      DIMENSION FEM(6)
      PDP=AL-PD
      IF(NP.EQ.1.OR.NP.EQ.4.OR.NP.EQ.5)GO TO 100
      FEM3=-PV*PD*PDP**2/AL**2
      FEM6=PV*PD**2*PDP/AL**2
      IF(NP.EQ.0)GO TO 110
      IF(NP.EQ.3.OR.NP.EQ.7)GO TO 90
C     J END PINNED
      FEM6=FEM6-0.5*FEM3
      FEM3=0.
      GO TO 110
   90 CONTINUE
C     K END PINNED
      FEM3 = FEM3 - 0.5*FEM6
      FEM6=0.
      GO TO 110
  100 CONTINUE
      FEM3=0.
      FEM6=0.
  110 CONTINUE
      FEM(2)=FEM(2)-PV*PDP/AL+(FEM3+FEM6)/AL
      FEM(5)=FEM(5)-PV*PD/AL-(FEM3+FEM6)/AL
      FEM(3)=FEM(3)+FEM3
      FEM(6)=FEM(6)+FEM6
      RETURN
      END
